@@ -6,7 +6,7 @@ use URI;
 use Digest::SHA;
 use List::Util qw/first/;
 
-requires qw/_authenticated_request ua/;
+requires qw/_add_authorization_header ua/;
 
 use namespace::autoclean;
 
@@ -20,10 +20,11 @@ around BUILDARGS => sub {
 
     my $args = $class->$orig(@_);
     my $oauth_urls = delete $args->{oauth_urls} || {
-        request_token_url  => "http://twitter.com/oauth/request_token",
-        authentication_url => "http://twitter.com/oauth/authenticate",
-        authorization_url  => "http://twitter.com/oauth/authorize",
-        access_token_url   => "http://twitter.com/oauth/access_token",
+        request_token_url  => "http://api.twitter.com/oauth/request_token",
+        authentication_url => "http://api.twitter.com/oauth/authenticate",
+        authorization_url  => "http://api.twitter.com/oauth/authorize",
+        access_token_url   => "http://api.twitter.com/oauth/access_token",
+        xauth_url          => "https://api.twitter.com/oauth/access_token",
     };
 
     return { %$oauth_urls, %$args };
@@ -33,7 +34,7 @@ has consumer_key    => ( isa => 'Str', is => 'ro', required => 1 );
 has consumer_secret => ( isa => 'Str', is => 'ro', required => 1 );
 
 # url attributes
-for my $attribute ( qw/authentication_url authorization_url request_token_url access_token_url/ ) {
+for my $attribute ( qw/authentication_url authorization_url request_token_url access_token_url xauth_url/ ) {
     has $attribute => (
         isa    => 'Str', is => 'rw', required => 1,
         # inflate urls to URI objects when read
@@ -77,7 +78,8 @@ sub get_authorization_url { return shift->_get_auth_url(authorization_url => @_)
 sub _make_oauth_request {
     my ($self, $type, %params) = @_;
 
-    my $request = Net::OAuth->request($type)->new(
+    my $class = $type =~ s/^\+// ? $type : Net::OAuth->request($type);
+    my $request = $class->new(
         version          => '1.0',
         consumer_key     => $self->{consumer_key},
         consumer_secret  => $self->{consumer_secret},
@@ -145,63 +147,76 @@ sub request_access_token {
     return (
         $self->access_token($res_param{oauth_token}),
         $self->access_token_secret($res_param{oauth_token_secret}),
+        $res_param{user_id},
+        $res_param{screen_name},
     );
 }
 
-override _authenticated_request => sub {
+around _prepare_request => sub {
+    my $orig = shift;
     my ($self, $http_method, $uri, $args, $authenticate) = @_;
 
-    delete $args->{source}; # not necessary with OAuth requests
-
-    my $msg;
-    if ( $authenticate && $self->authorized ) {
-        local $Net::OAuth::SKIP_UTF8_DOUBLE_ENCODE_CHECK = 1;
-
-        my $is_multipart = first { ref } %$args;
-
-        my $request = $self->_make_oauth_request(
-            'protected resource',
-            request_url    => $uri,
-            request_method => $http_method,
-            token          => $self->access_token,
-            token_secret   => $self->access_token_secret,
-            extra_params   => $is_multipart ? {} : $args,
-        );
-
-        if ( $http_method =~ /^GET|DELETE$/ ) {
-            $msg = HTTP::Request->new($http_method, $request->to_url);
-        }
-        elsif ( $http_method eq 'POST' ) {
-            $msg = $is_multipart
-                 ? POST($request->request_url,
-                        Authorization => $request->to_authorization_header,
-                        Content_Type  => 'form-data',
-                        Content       => [ %$args ],
-                   )
-                 : POST($uri, Content => $request->to_post_body)
-                 ;
-        }
-        else {
-            croak "unexpected http_method: $http_method";
-        }
-    }
-    elsif ( $http_method =~ /^GET|DELETE$/ ) {
-        $self->_encode_args($args);
-        $uri->query_form($args);
-        $args = {};
-        $msg = HTTP::Request->new($http_method, $uri);
-    }
-    elsif ( $http_method eq 'POST' ) {
-        $self->_encode_args($args);
-        $msg = POST($uri, $args);
-    }
-    else {
-        croak "unexpected http_method: $http_method";
-    }
-
-    return $self->ua->request($msg);
+    delete $args->{source};
+    $orig->(@_);
 };
 
+override _add_authorization_header => sub {
+    my ($self, $msg) = @_;
+
+    return unless $self->authorized;
+
+    my $uri = $msg->uri->clone;
+    $uri->query($msg->content) if $msg->content_type eq 'application/x-www-form-urlencoded';
+    my $args = { $uri->query_form };
+
+    local $Net::OAuth::SKIP_UTF8_DOUBLE_ENCODE_CHECK = 1;
+
+    my $request = $self->_make_oauth_request(
+        'protected resource',
+        request_url    => $msg->uri,
+        request_method => $msg->method,
+        token          => $self->access_token,
+        token_secret   => $self->access_token_secret,
+        extra_params   => $args,
+    );
+
+    $msg->header(authorization => $request->to_authorization_header);
+};
+
+sub xauth {
+    my ( $self, $username, $password ) = @_;
+
+    my @args = (
+        x_auth_username => $username,
+        x_auth_password => $password,
+        x_auth_mode     => 'client_auth',
+    );
+
+    my $uri = $self->xauth_url;
+    my $request = $self->_make_oauth_request(
+        'XauthAccessToken',
+        request_url    => $uri,
+        request_method => 'POST',
+        @args,
+    );
+
+    my $res = $self->ua->request(
+        POST $uri, \@args, Authorization => $request->to_authorization_header);
+    die "POST $uri failed: ".$res->status_line
+        unless $res->is_success;
+
+    # reuse $uri to extract parameters from content
+    $uri->query($res->content);
+    my %res_param = $uri->query_form;
+
+    return (
+        $self->access_token($res_param{oauth_token}),
+        $self->access_token_secret($res_param{oauth_token_secret}),
+        $res_param{user_id},
+        $res_param{screen_name},
+    );
+}
+
 # shortcuts defined in early releases
 # DEPRECATED
 
@@ -299,7 +314,7 @@ Here's how to authorize users as a desktop app mode:
       my $pin = <STDIN>; # wait for input
       chomp $pin;
 
-      my($access_token, $access_token_secret) = $nt->request_access_token(verifier => $pin);
+      my($access_token, $access_token_secret, $user_id, $screen_name) = $nt->request_access_token(verifier => $pin);
       save_tokens($access_token, $access_token_secret); # if necessary
   }
 
@@ -338,7 +353,7 @@ secret to upgrade the request token to access token.
       $nt->request_token($cookie{token});
       $nt->request_token_secret($cookie{token_secret});
 
-      my($access_token, $access_token_secret)
+      my($access_token, $access_token_secret, $user_id, $screen_name)
           = $nt->request_access_token(verifier => $verifier);
 
       # Save $access_token and $access_token_secret in the database associated with $c->user
@@ -373,15 +388,24 @@ Note that the credentials may be wrong and so the request may fail.
 
 =item request_access_token(verifier => $verifier)
 
-Request the access token and access token secret for this user. You must pass
-the PIN# (for desktop applications) or the C<ouath_verifier> value, provided as
-a parameter to the oauth callback (for web applications) as C<$verifier>.
+Request the access token, access token secret, user id and screen name for
+this user. You must pass the PIN# (for desktop applications) or the
+C<ouath_verifier> value, provided as a parameter to the oauth callback
+(for web applications) as C<$verifier>.
 
 The user must have authorized this app at the url given by C<get_authorization_url> first.
 
-Returns the access token and access token secret but also sets them internally
-so that after calling this method, you can immediately call API methods
-requiring authentication.
+Returns the access_token, access_token_secret, user_id, and screen_name in a
+list.  Also sets them internally so that after calling this method, you can
+immediately call API methods requiring authentication.
+
+=item xauth($username, $password)
+
+Exchanges the C<$username> and C<$password> for access tokens.  This method has
+the same return value as C<request_access_token>: access_token, access_token_secret,
+user_id, and screen_name in a list. Also, like C<request_access_token>, it sets
+the access_token and access_secret, internally, so you can immediately call API
+methods requiring authentication.
 
 =item get_authorization_url(callback => $callback_url)
 
